@@ -1,9 +1,9 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2023 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2023-2024 -- leonerd@leonerd.org.uk
 
-package meta 0.003;
+package meta 0.004;
 
 use v5.14;
 use warnings;
@@ -11,6 +11,11 @@ use warnings;
 require XSLoader;
 XSLoader::load( __PACKAGE__, our $VERSION );
 
+# Hackery to make warnings::warnif callable from XS, on perls too old to have
+# warnif_at_level
+$^V ge v5.28 or
+   *warnif_trampoline = sub { warnings::warnif(@_); };
+
 =head1 NAME
 
 C<meta> - meta-programming API
@@ -48,6 +53,18 @@ versions. Once a workable API shape has been found, it is hoped that this
 module will eventually become dual-life and shipped as part of Perl core, as
 the implementation for PPC 0022. See the link in the L</SEE ALSO> section.
 
+I<Since version 0.003_002> all the entry-point functions and constructors in
+this module will provoke warnings in the C<meta::experimental> category. They
+can be silenced by
+
+   use meta;
+   no warnings 'meta::experimental';
+
+I<Since version 0.003_003> the various C<undef>-returning variants of
+C<can_...> like functions or methods have aliases called C<try_get_...> as
+this may be a more suitable naming pattern. The older C<can_...> variants are
+likely to be removed soon.
+
 =cut
 
 =head1 FUNCTIONS
@@ -59,6 +76,8 @@ the implementation for PPC 0022. See the link in the L</SEE ALSO> section.
 Returns a metapackage reference representing the given package name, creating
 it if it did not previously exist.
 
+An alternative to C<< meta::package->get >> in a plain function style.
+
 =head2 get_this_package
 
    $metapkg = meta::get_this_package;
@@ -75,7 +94,18 @@ C<BEGIN> or loading time. Equivalent to but more efficient than the following:
 
 =cut
 
-=head1 METHODS ON METAPACKAGES
+=head1 METHODS ON C<meta::package>
+
+=head2 get
+
+   $metapkg = meta::package->get( $pkgname );
+
+I<Since version 0.003_001.>
+
+Returns a metapackage reference representing the given package name, creating
+it if it did not previously exist.
+
+An alternative to C<meta::get_package> in an object constructor style.
 
 =head2 name
 
@@ -90,8 +120,9 @@ Returns the name of the package being represented.
 Returns a metaglob reference representing the given symbol name within the
 package, if it exists. Throws an exception if not.
 
-=head2 can_glob
+=head2 try_get_glob, can_glob
 
+   $metaglob = $metapkg->try_get_glob( $name );
    $metaglob = $metapkg->can_glob( $name );
 
 Similar to L</get_glob> but returns undef if the glob does not exist.
@@ -105,11 +136,12 @@ package. The symbol name should include the leading sigil; one of the
 characters C<*>, C<$>, C<@>, C<%> or C<&>. Throws an exception if the symbol
 does not exist.
 
-=head2 can_symbol
+=head2 try_get_symbol, can_symbol
 
+   $metasym = $metapkg->try_get_symbol( $name );
    $metasym = $metapkg->can_symbol( $name );
 
-Similar to L</get_symbol> but returns undef if the symbol does ont exist.
+Similar to L</get_symbol> but returns undef if the symbol does not exist.
 
 =head2 add_symbol
 
@@ -120,12 +152,6 @@ will refer to the item given by reference, whose type must match the sigil
 of the symbol name. Returns a metasymbol reference as per L</get_symbol>. If
 a symbol already existed of the given name then an exception is thrown.
 
-To only conditionally add a symbol if it doesn't already exist, test for it
-first by using L</can_symbol>:
-
-   $metapkg->can_symbol( '$variable' ) or
-      $metapkg->add_symbol( '$variable', \my $tmp );
-
 I<Note> that this does not create a copy of a variable, but stores an alias
 to the referred item itself within the symbol table.
 
@@ -134,6 +160,23 @@ to the referred item itself within the symbol table.
    push @array, "more", "values";
    # these values are now visible in the @things array
 
+If adding a scalar, array or hash variable, the C<$valueref> argument is
+optional. If not provided then a new, blank variable of the correct type will
+be created.
+
+=head2 get_or_add_symbol
+
+   $metasym = $metapkg->get_or_add_symbol( $name, $valueref );
+
+I<Since version 0.003_003.>
+
+Similar to L</get_symbol> but creates a new symbol if it didn't already exist
+as per L</add_symbol>.
+
+Note that if the symbol did already exist it is returned and C<$valueref> will
+be ignored. The symbol will not be modified in that case to point to the value
+referred to instead.
+
 =head2 remove_symbol
 
    $metapkg->remove_symbol( $name );
@@ -143,9 +186,9 @@ the last item in the glob then the glob too is removed from the package. If
 the named symbol did not previously exist then an exception is thrown.
 
 To only conditionally remove a symbol if it already exists, test for it first
-by using L</can_symbol>:
+by using L</try_get_symbol>:
 
-   $metapkg->can_symbol( '$variable' ) and
+   $metapkg->try_get_symbol( '$variable' ) and
       $metapkg->remove_symbol( '$variable' );
 
 =cut
@@ -171,12 +214,37 @@ Returns a regular Perl reference to the symbol being represented.
 
 =cut
 
-=head1 METHODS ON METAGLOBS
+=head1 METHODS ON C<meta::glob>
 
 =cut
 
 @meta::glob::ISA = qw( meta::symbol );
 
+=head2 get
+
+   $metaglob = meta::glob->get( $globname );
+
+I<Since version 0.003_001.>
+
+Returns a metaglob reference representing the given symbol from the symbol
+table from a fully-qualified name, if it exists. Throws an exception if not.
+
+=head2 try_get
+
+   $metaglob = meta::glob->try_get( $globname );
+
+I<Since version 0.003_003.>
+
+Similar to L</get> but returns undef if the given symbol does not exist.
+
+=head2 get_or_add
+
+   $metaglob = meta::glob->get_or_add( $globname );
+
+I<Since version 0.003_003.>
+
+Similar to L</get> but creates the symbol if it didn't already exist.
+
 =head2 name
 
    $name = $metaglob->basename;
@@ -193,7 +261,7 @@ Returns the name of the glob I<within its package>.
 Returns a metasymbol reference representing the symbol in the given slot of
 the glob, if it exists. Throws an exception if not.
 
-=head2 can_scalar, can_array, ...
+=head2 try_get_scalar, try_get_array, ...
 
 Similar to L</get_scalar>, L</get_array>, etc... but returns undef if the
 given slot does not exist.
